@@ -1,7 +1,7 @@
 library(shiny)
 
 # Define UI for dataset viewer application
-shinyUI(fluidPage(
+fluidPage(
   
   # Application title
   titlePanel("Reactivity"),
@@ -22,7 +22,7 @@ shinyUI(fluidPage(
     
     
     # Show the caption, a summary of the dataset and an HTML 
-	 # table with the requested number of observations
+    # table with the requested number of observations
     mainPanel(
       h3(textOutput("caption", container = span)),
       
@@ -31,4 +31,4 @@ shinyUI(fluidPage(
       tableOutput("view")
     )
   )
-))
+)
