@@ -1,4 +1,4 @@
-shinyServer(function(input, output) {
+function(input, output) {
   datasetInput <- reactive({
     switch(input$dataset,
            "rock" = rock,
@@ -12,10 +12,10 @@ shinyServer(function(input, output) {
   
   output$downloadData <- downloadHandler(
     filename = function() { 
-		 paste(input$dataset, '.csv', sep='') 
-	 },
+      paste(input$dataset, '.csv', sep='')
+    },
     content = function(file) {
       write.csv(datasetInput(), file)
     }
   )
-})
+}
